// Shake it to wake it program main board
// Operation of the vacuum valves and the motor


// Include any librarys
#include <Servo.h>

// Define our inputs
#define in_pump  A0
#define in_vvalve A1
#define in_vvent A2
#define in_shakeL A3
#define in_shakeH A4
#define in_tilt A5
#define in_toplim 12
#define in_botlim 13

// Define some outputs
#define out_pump 2
#define out_vvent 4
#define out_tilt 7

// Name each of the servos
Servo shake;
Servo vvalve;

// Define the limits of each servo
int shake_hi = 100;
int shake_mid = 70;
int shake_lo = 0;
int vvalve_hi = 180;
int vvalve_lo = 0;

// Memory for if the search commands were issued
int topFind = 0;
int botFind = 0;

void setup() {
  // Attach each of the servo objects to a pin
  shake.attach(3);
  vvalve.attach(5);
  
  // Set input pin modes
  pinMode(in_vvalve, INPUT);
  pinMode(in_vvent, INPUT);
  pinMode(in_shakeL, INPUT);
  pinMode(in_shakeH, INPUT);
  pinMode(in_tilt, INPUT);
  pinMode(in_pump, INPUT);

  // Output pin modes
  pinMode(out_pump, OUTPUT);
  pinMode(out_vvent, OUTPUT);
  pinMode(out_tilt, OUTPUT);

  // Set up serial
  Serial.begin(9600);
  Serial.println("Start");

}

void loop() {
  // Check each input and respond
  if (digitalRead(in_vvalve)) {
    ssweep(vvalve, vvalve_hi);
  }
  else {
    ssweep(vvalve, vvalve_lo);
  }

  if (digitalRead(in_pump)) {
    digitalWrite(out_pump, HIGH);
  }
  else {
    digitalWrite(out_pump, LOW);
  }

  if (digitalRead(in_vvent)) {
    digitalWrite(out_vvent, HIGH);
  }
  else {
    digitalWrite(out_vvent, LOW);
  }

  if (digitalRead(in_tilt)) {
    digitalWrite(out_tilt, HIGH);
  }
  else {
    digitalWrite(out_tilt, LOW);
  }

  if ((digitalRead(in_shakeL) == 1) && (digitalRead(in_shakeH) == 0)) {
    ssweep(shake, shake_hi);
    topFind = 0;
    botFind = 0;
  }
  else if ((digitalRead(in_shakeL) == 0) && (digitalRead(in_shakeH) == 0)){
    ssweep(shake, shake_lo);
    topFind = 0;
    botFind = 0;
  }

  if ((digitalRead(in_shakeL) == 0) && (digitalRead(in_shakeH) == 1) && (topFind == 0) ) {
    topFind = 1;
    findTop();
  }

  if ((digitalRead(in_shakeL) == 1) && (digitalRead(in_shakeH) == 1) && (botFind == 0) ) {
    botFind = 1;
    findBot();
  }


  // Debug output
  Serial.println("Inputs");
  Serial.print("--> in_pump = ");
  Serial.println(digitalRead(in_pump));
  Serial.print("--> in_vvalve = ");
  Serial.println(digitalRead(in_vvalve));
  Serial.print("--> in_vvent = ");
  Serial.println(digitalRead(in_vvent));
  Serial.print("--> in_shakeL = ");
  Serial.println(digitalRead(in_shakeL));
  Serial.print("--> in_shakeH = ");
  Serial.println(digitalRead(in_shakeH));
  Serial.print("--> in_tilt = ");
  Serial.println(digitalRead(in_tilt));
  Serial.print("--> Top Lim = ");
  Serial.println(digitalRead(in_toplim));
  Serial.print("--> Bot Lim = ");
  Serial.println(digitalRead(in_botlim));
}

// Seak the top position
void findTop() {
  // Finds the top spot

  // If alread there exit
  if (digitalRead(in_toplim) == 1) {
    return;
  }

  // Run the motor slow until it hits the switch. Delay, then stop.
  ssweep(shake, shake_mid);

  while (digitalRead(in_toplim) != 1) {
    // Wait
  }

  // Now delay then stop
  delay(5);
  ssweep(shake, shake_lo);
}

// Seak the bottom position
void findBot() {
  // Finds the bottom spot

  // If alread there exit
  if (digitalRead(in_botlim) == 1) {
    return;
  }

  // Run the motor slow until it hits the switch. Delay, then stop.
  ssweep(shake, shake_mid);

  while (digitalRead(in_botlim) != 1) {
    // Wait
  }

  // Now delay then stop
  delay(5);
  ssweep(shake, shake_lo);
}

// Servo sweeping function
void ssweep(Servo &id, int to) {

  // Declarations
  int from;

  // Get where it currently is
  from = id.read();

  // Check if we are already there
  if (to == from) {
    // Don't do anything
    return;
  }

  Serial.println("Move Start");

  // Now move it nicely
  if (from < to) {
    for (int i=from; i <= to; i++){
      id.write(i);
      delay(5);
    }
  }
  else {
    for (int i=from; i >= to; i--){
      id.write(i);
      delay(5);
    }
  }

  Serial.println("Move Stop");
}

